"use strict";
// Some functions taken from FoalTS
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.verifyPassword = exports.hashPassword = exports.generateHexToken = void 0;
const crypto_1 = require("crypto");
const util_1 = require("util");
const assert_1 = require("assert");
function generateHexToken(size) {
    return (0, crypto_1.randomBytes)(size).toString('hex');
}
exports.generateHexToken = generateHexToken;
/**
 * Legacy function to hash passwords. Only kept for backward compatibility.
 * @param password
 */
function parsePassword(password) {
    return __awaiter(this, void 0, void 0, function* () {
        const salt = (yield (0, util_1.promisify)(crypto_1.randomBytes)(16)).toString('hex');
        const iterations = 100000;
        const keylen = 64;
        const digest = 'sha256';
        const derivedKey = yield (0, util_1.promisify)(crypto_1.pbkdf2)(password, salt, iterations, keylen, digest);
        return `pbkdf2_${digest}$${iterations}$${salt}$${derivedKey.toString('hex')}`;
    });
}
/**
 * Hash a password using the PBKDF2 algorithm.
 *
 * Configured to use PBKDF2 + HMAC + SHA256.
 * The result is a 64 byte binary string (or hex if the legacy option is true).
 *
 * The random salt is 16 bytes long.
 * The number of iterations is 150000.
 * The length key is 32 bytes long.
 *
 * @export
 * @param {string} plainTextPassword - The password to hash.
 * @param {{ legacy?: boolean }} [options={}]
 * @returns {Promise<string>} The derived key with the algorithm name, the number of iterations and the salt.
 */
function hashPassword(plainTextPassword, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        // apply basic safeguard
        if (plainTextPassword.trim().length < 6) {
            throw new Error('Password invalid: too short');
        }
        if (options.legacy) {
            return parsePassword(plainTextPassword);
        }
        const saltBuffer = yield (0, util_1.promisify)(crypto_1.randomBytes)(16);
        const iterations = 150000;
        const keylen = 32;
        const digest = 'sha256';
        const derivedKeyBuffer = yield (0, util_1.promisify)(crypto_1.pbkdf2)(plainTextPassword, saltBuffer, iterations, keylen, digest);
        const salt = saltBuffer.toString('base64');
        const derivedKey = derivedKeyBuffer.toString('base64');
        return `pbkdf2_${digest}$${iterations}$${salt}$${derivedKey}`;
    });
}
exports.hashPassword = hashPassword;
/**
 * Compare a plain text password and a hash to see if they match.
 *
 * @export
 * @param {string} plainTextPassword - The password in clear text.
 * @param {string} passwordHash - The password hash generated by the `hashPassword` function.
 * @param {{ legacy?: boolean }} [options={}]
 * @returns {Promise<boolean>} True if the hash and the password match. False otherwise.
 */
function verifyPassword(plainTextPassword, passwordHash, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const legacy = options.legacy || false;
        const [algorithm, iterations, salt, derivedKey] = passwordHash.split('$');
        (0, assert_1.strictEqual)(algorithm, 'pbkdf2_sha256', 'Invalid algorithm.');
        (0, assert_1.strictEqual)(typeof iterations, 'string', 'Invalid password format.');
        (0, assert_1.strictEqual)(typeof salt, 'string', 'Invalid password format.');
        (0, assert_1.strictEqual)(typeof derivedKey, 'string', 'Invalid password format.');
        (0, assert_1.strictEqual)(isNaN(parseInt(iterations, 10)), false, 'Invalid password format.');
        const saltBuffer = Buffer.from(salt, legacy ? 'hex' : 'base64');
        const derivedKeyBuffer = Buffer.from(derivedKey, legacy ? 'hex' : 'base64');
        const digest = 'sha256'; // TODO: depends on the algorthim var
        const password = yield (0, util_1.promisify)(crypto_1.pbkdf2)(plainTextPassword, legacy ? saltBuffer.toString('hex') : saltBuffer, parseInt(iterations, 10), derivedKeyBuffer.length, digest);
        return (0, crypto_1.timingSafeEqual)(password, derivedKeyBuffer);
    });
}
exports.verifyPassword = verifyPassword;
